from django.shortcuts import get_object_or_404, render, redirect
from django.utils.crypto import get_random_string
from django.http import HttpResponse, HttpResponseForbidden
from django.core.paginator import Paginator
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.core.files.storage import FileSystemStorage
from datetime import datetime
import xlwt

from products.models import Product
from profiling.models import ProfilingJob
from transport.models import Driver, DriverRating
from account.models import Account
from quotations.models import CustomerQuoteInfo, Inquiry, Quotation
from .models import Dispatch, DispatchImage
from django.db.models import Q

@login_required
def quotation_dispatched_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.") 

    status_filter = request.GET.get('status', None)
    print(status_filter)
 
    production_dispatches = Dispatch.objects.all()
    if status_filter:
        production_dispatches = production_dispatches.filter(quotation__status=status_filter)

    return render(request, 'dispatch/quotation_dispatched_list.html', {'production_dispatches': production_dispatches})
@login_required
def awaiting_dispatched_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    production_dispatches = Dispatch.objects.filter(dispatch_type='Production')
    return render(request, 'dispatch/quotation_dispatched_list.html', {'production_dispatches': production_dispatches})


@login_required
def quotation_arrived_dispatch_list(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    production_dispatches = Dispatch.objects.filter(dispatch_type='Production', quotation__status='Arrived')
    return render(request, 'dispatch/quotation_arrived_dispatch_list.html',
                  {'production_dispatches': production_dispatches})


from django.shortcuts import render
from django.http import HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator
from .models import Dispatch, Quotation, Driver  # Adjust import based on your app structure
from datetime import datetime
from django.contrib import messages

from django.shortcuts import render
from django.http import HttpResponseForbidden
from django.contrib.auth.decorators import login_required
from django.core.paginator import Paginator
from .models import Driver, Quotation, Dispatch  # Adjust the import based on your app structure
from datetime import datetime
from django.contrib import messages

@login_required
def index(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    drivers = Driver.objects.all()
    track_number = request.GET.get('track_number', '')
    quotation_id = request.GET.get('quotation_id', '')
    start_date = request.GET.get('start_date', '')
    end_date = request.GET.get('end_date', '')
    status = request.GET.get('status', '')  # New status filter

    if quotation_id:
        messages.success(request, "Successfully Dispatched the Quotation")
        Quotation.objects.filter(id=quotation_id).update(status="Dispatched")

    # Start with dispatch entries
    dispatch_entries = Dispatch.objects.select_related('quotation').filter(status='Dispatched').order_by('-dispatch_date')

    # Apply filters based on GET request parameters
    if track_number:
        dispatch_entries = dispatch_entries.filter(track_number__icontains=track_number)

    if status:  # Filter by status if provided
        dispatch_entries = dispatch_entries.filter(status=status)

    if start_date and end_date:
        start_date_obj = datetime.strptime(start_date, "%Y-%m-%d").date()
        end_date_obj = datetime.strptime(end_date, "%Y-%m-%d").date()
        dispatch_entries = dispatch_entries.filter(dispatch_date__range=(start_date_obj, end_date_obj))

    paginator = Paginator(dispatch_entries, 10)
    page = request.GET.get('page')
    dispatch_entries_page = paginator.get_page(page)

    data = {
        'dispatch_entries': dispatch_entries_page,
        'DISPATCH_STATUS_CHOICES': Dispatch.DISPATCH_STATUS_CHOICES,
        'track_number': track_number,
        'start_date': start_date,
        'end_date': end_date,
        'drivers': drivers,
        'selected_status': status  # Pass the selected status back to the template
    }

    return render(request, 'sales/dispatch/index.html', data)

@login_required
def awaiting_dispatch(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    drivers = Driver.objects.all()
    quotations = Quotation.objects.filter(status="Produced")
    data = {
        "production_quotations": quotations,
        "drivers": drivers,
        'is_admin': request.user.is_admin_user
    }
    return render(request, "dispatch/awaiting_dispatch.html", data)


from django.shortcuts import render, get_object_or_404
from django.http import HttpResponseForbidden
from django.core.paginator import Paginator
from django.contrib import messages
from django.utils.crypto import get_random_string
from datetime import datetime
from .models import Dispatch, Quotation, Driver, Account
from django.shortcuts import render, get_object_or_404, redirect
from django.http import HttpResponseForbidden
from django.contrib import messages
from django.utils.crypto import get_random_string
from datetime import datetime
from .models import Dispatch, Quotation, Driver, Vehicle
from django.contrib.auth.decorators import login_required

from django.shortcuts import render, get_object_or_404, redirect
from django.http import HttpResponseForbidden
from django.contrib import messages
from django.utils.crypto import get_random_string
from datetime import datetime
from .models import DispatchProduct, Quotation, Driver, Vehicle
from django.contrib.auth.decorators import login_required
from django.db.models import Q
from .models import Quotation

 
@login_required
def dispatch(request):  
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    selected_status = request.GET.get('status', '')

    if request.user.is_admin_user or request.user.is_super_user: 
        quotations = Quotation.objects.filter(
            Q(payment_status="PAID"),
            ~Q(status="In Production")   
        ).order_by("-id")
    else: 
        quotations = Quotation.objects.filter(
            Q(payment_status="PAID"),
            ~Q(status="In Production"),  
            seller=request.user
        ).order_by("-id")


    if selected_status:   
        quotations = quotations.filter(status=selected_status)

    drivers = Driver.objects.all()
    vehicles = Vehicle.objects.all()

    if request.method == 'POST':
        action = request.POST.get('action')

        print(action)

        if action == 'create_dispatch_product':
            quotation_id = request.POST.get('quotation_id')
            driver_id = request.POST.get('driver_id')
            vehicle_id = request.POST.get('vehicle_id')
            transport_cost = request.POST.get('transport_cost')
            dispatch_date_str = request.POST.get('dispatch_date')
            expected_delivery_date_str = request.POST.get('expected_delivery_date')

            dispatch_date = datetime.strptime(dispatch_date_str, '%Y-%m-%d')
            expected_delivery_date = datetime.strptime(expected_delivery_date_str, '%Y-%m-%d')
            dispatched_by = request.user

            driver = get_object_or_404(Driver, pk=driver_id)
            vehicle = get_object_or_404(Vehicle, pk=vehicle_id)

            quotation = get_object_or_404(Quotation, pk=quotation_id)

            # Check if all products in the quotation are eligible for dispatch
            inquiries = Inquiry.objects.filter(quotation=quotation)

            # Condition 1: Ensure all "Production Items" are fully produced
            incomplete_production = inquiries.filter(
                product__productType="Production Item",
                production_status__in=["Pending", "In Production"]   
            )

            if incomplete_production.exists():
                messages.error(request, "Cannot dispatch. Some Production Items are not fully produced.")
                return render(request, 'sales/dispatch/index.html', {
                    'quotations': quotations,
                    'drivers': drivers,
                    'vehicles': vehicles
                })

            # Condition 2: Ensure the quotation is fully paid
            if quotation.payment_status != 'PAID':
                messages.error(request, "Cannot dispatch. The quotation is not fully paid.")
                return render(request, 'sales/dispatch/index.html', {
                    'quotations': quotations,
                    'drivers': drivers,
                    'vehicles': vehicles
                })

            # If all checks pass, create the dispatch product
            dispatch_product = DispatchProduct(
                driver=driver,
                vehicle=vehicle,
                transport_cost=transport_cost,
                dispatch_date=dispatch_date,
                expected_delivery_date=expected_delivery_date,
                quotation_id=quotation_id,
                dispatched_by=dispatched_by,
                status="Dispatched"
            )
            dispatch_product.save()

            # Update quotation status All inquiries processed and quantity_received updated successfully
            quotation.status = "Shipped"
            quotation.save()

            messages.success(request, f"Successfully created dispatch product for quotation ID {quotation_id}.")
            return render(request, 'sales/dispatch/index.html', {
                'quotations': quotations,
                'drivers': drivers,
                'vehicles': vehicles
            })

    return render(request, 'sales/dispatch/index.html', {
        'quotations': quotations,
        'drivers': drivers,
        'vehicles': vehicles,
        'selected_status': selected_status  # Pass the selected status back to the template
    })

from django.shortcuts import get_object_or_404
from django.http import JsonResponse
from django.core.exceptions import ObjectDoesNotExist 
from django.db import IntegrityError


import logging
from django.http import JsonResponse
from rest_framework.decorators import api_view
from django.core.exceptions import ObjectDoesNotExist
from django.db import IntegrityError 

# Configure logging (optional, you can adjust the level and format)
logger = logging.getLogger(__name__)

from django.http import JsonResponse
from django.shortcuts import get_object_or_404
from django.core.exceptions import ObjectDoesNotExist

@api_view(['POST'])
def update_delivery_docs_and_rate_driver(request): 
    # Log the received request (use print for logging)
    print(f"Received request to update delivery docs and rate driver: {request.data}")

    # Extracting data from the request
    dispatch_product_id = request.data.get('dispatch_product_id')
    driver_rating = request.data.get('driver_rating')
    driver_comments = request.data.get('driver_comments', '')
    documents = request.FILES.getlist('documents')

    # Log the extracted data (excluding file data for security reasons)
    print(f"Dispatch Product ID: {dispatch_product_id}")
    print(f"Driver Rating: {driver_rating}")
    print(f"Driver Comments: {driver_comments}")
    print(f"Number of documents uploaded: {len(documents)}")

    try:
        # Check if DispatchProduct exists
        dispatch_product = get_object_or_404(DispatchProduct, quotation_id=dispatch_product_id)
        
        # Update the driver rating
        driver = dispatch_product.driver
        new_rating = int(driver_rating)

        # Log the driver information
        print(f"Updating driver rating for driver ID {driver.id}")

        # Update the rating for the driver
        driver.update_rating(new_rating)

        # Save the rating to DriverRating model
        driver_rating_instance = DriverRating.objects.create(
            driver=driver,
            rating=new_rating,
            comments=driver_comments
        )
        
        # Handle the uploaded documents (e.g., storing them in the database or file system)
        for doc in documents:
            # Log the document being processed
            print(f"Processing document: {doc.name}")  # Log the document name
            # Add logic to save documents if needed
            # Example: save the file to a specific model or file system
            pass  # Replace this with actual logic to store documents

        # Mark the associated quotation as 'Received'
        quotation = dispatch_product.quotation  # Assuming `dispatch_product` has a `quotation` field
        quotation.status = 'Received'
        quotation.save()  # Save the updated status

        # If everything goes fine, log success and return success response
        print("Delivery documents and driver rating updated successfully, and quotation marked as 'Received'.")
        return JsonResponse({'message': 'Delivery documents and driver rating updated successfully, and quotation marked as "Received"'}, status=200)

    except ObjectDoesNotExist as e:
        print(f"Error: Related object not found: {str(e)}")
        return JsonResponse({'error': f'Related object not found: {str(e)}'}, status=404)
    except Exception as e:
        print(f"Error: An unexpected error occurred: {str(e)}")
        return JsonResponse({'error': f'An unexpected error occurred: {str(e)}'}, status=500)


from django.shortcuts import get_object_or_404, render
from django.http import HttpResponse # Assuming you have Document model for files
from django.core.exceptions import ObjectDoesNotExist

from django.shortcuts import get_object_or_404, render
from django.http import HttpResponse
from django.core.exceptions import ObjectDoesNotExist 

def dispatch_details(request, quotation_id):
    try:
        # Fetch the Quotation object based on quotation_id
        quotation = get_object_or_404(Quotation, id=quotation_id)

        # Get associated DispatchProduct
        dispatch_product = DispatchProduct.objects.filter(quotation_id=quotation_id).first()

        if not dispatch_product:
            return HttpResponse("Dispatch Product not found for this quotation.", status=404)

        # Get associated Inquiries for the quotation
        inquiries = Inquiry.objects.filter(quotation=quotation)

        # Get the Driver associated with the DispatchProduct
        driver = dispatch_product.driver if dispatch_product else None

        # Get the dispatched_by user account (assuming DispatchProduct has dispatched_by as a ForeignKey)
        dispatched_by = dispatch_product.dispatched_by if dispatch_product else None

        # Fetch the CustomerQuoteInfo related to the quotation to get customer details
        customer_info = CustomerQuoteInfo.objects.filter(quotation=quotation).first() 
        print(customer_info)

        # Prepare context data for rendering the template
        context = {
            "quotation": quotation,
            "inquiries": inquiries,
            "driver": driver,
            "dispatched_by": dispatched_by,
            "customer": customer_info
        }

        # Render the data in a template
        return render(request, 'sales/dispatch/dispatch_details.html', context)

    except ObjectDoesNotExist as e:
        print(f"Error: Related object not found: {str(e)}")
        return HttpResponse(f"Error: Related object not found: {str(e)}", status=404)
    except Exception as e:
        print(f"Error: An unexpected error occurred: {str(e)}")
        return HttpResponse(f"Error: An unexpected error occurred: {str(e)}", status=500)

from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from django.shortcuts import get_object_or_404
from datetime import datetime 

from rest_framework.response import Response
from rest_framework.decorators import api_view
from rest_framework import status
from datetime import datetime 
  
from django.db import transaction 

@api_view(['POST'])
def submit_credit_note(request):
    print("Received data:", request.data)

    # Extract data from request
    quotation_id = request.data.get('quotation_id')
    driver_id = request.data.get('driver_id')
    vehicle_id = request.data.get('vehicle_id')
    transport_cost = request.data.get('transport_cost')
    dispatch_date_str = request.data.get('dispatch_date')
    expected_delivery_date_str = request.data.get('expected_delivery_date')
    inquiries = request.data.get('inquiries', [])
    dispatched_by_user_id = request.data.get('dispatched_by')

    # Convert dispatch and delivery dates to datetime objects
    try:
        dispatch_date = datetime.strptime(dispatch_date_str, '%Y-%m-%d')
        expected_delivery_date = datetime.strptime(expected_delivery_date_str, '%Y-%m-%d')
    except ValueError:
        return Response({"error": "Invalid date format. Use YYYY-MM-DD."}, status=status.HTTP_400_BAD_REQUEST)

    # Retrieve the Account instance associated with the dispatched_by user
    try:
        dispatched_by_account = Account.objects.get(id=dispatched_by_user_id)
    except Account.DoesNotExist:
        return Response({"error": "Account not found for the provided user ID."}, status=status.HTTP_404_NOT_FOUND)

    # Begin a transaction to ensure data consistency
    with transaction.atomic():
        # Retrieve the Quotation instance
        quotation = get_object_or_404(Quotation, id=quotation_id)
        # Fetch inquiries associated with the quotation from the database
        inquiries_in_quotation = Inquiry.objects.filter(quotation=quotation)

        # Initialize a flag to check incomplete production items
        incomplete_production = []

        # Iterate through the inquiries to find incomplete production items
        for inquiry in inquiries:
            product_type = inquiry.get('product_type')
            production_status = inquiry.get('production_status')

            if product_type == "Production Item" and production_status in ["Pending", "In Production"]:
                incomplete_production.append(inquiry)

        # If there are incomplete production items, return an error response
        if incomplete_production:
            return Response({
                "error": "Cannot create credit note. Some Production Items are not fully produced.",
                "incomplete_items": incomplete_production  # Optionally return the details of incomplete items
            }, status=status.HTTP_400_BAD_REQUEST)

        

        # Condition 2: Ensure the quotation is fully paid
        if quotation.payment_status != 'PAID':
            return Response({
                "error": "Cannot create credit note. The quotation is not fully paid."
            }, status=status.HTTP_400_BAD_REQUEST)

        # Get or create the DispatchProduct
        dispatch_product, created = DispatchProduct.objects.get_or_create(
            quotation_id=quotation_id,
            defaults={
                'driver_id': driver_id,
                'vehicle_id': vehicle_id,
                'transport_cost': transport_cost,
                'dispatch_date': dispatch_date,
                'expected_delivery_date': expected_delivery_date,
                'dispatched_by': dispatched_by_account,
                'status': "Dispatched"
            }
        )

        # If the DispatchProduct was newly created, set the Quotation status to "Shipped"
        if created:
            quotation.status = "Shipped"
            quotation.save()

        # Process each inquiry in the list and update quantity_received
        for inquiry_data in inquiries:
            inquiry_id = inquiry_data.get('inquiry_id')
            quantity_received = inquiry_data.get('quantity_received')

            # Ensure Inquiry exists
            inquiry = get_object_or_404(Inquiry, id=inquiry_id)
            quantity_available = inquiry.quantity

            # Check if the received quantity is valid
            if quantity_received < 0 or quantity_received > quantity_available:
                raise ValueError(f"Received quantity must be between 0 and {quantity_available} for Inquiry ID: {inquiry_id}")
 
            # Update the inquiry if the quantity is valid 
            inquiry.quantity_received = quantity_received
            
            if inquiry.product.productType == "Trading Item":

                # Use get() to retrieve a single product instance
                product = Product.objects.get(product_id=inquiry.product.product_id)

                # Update the quantity_in_stock
                product.quantity_in_stock -= quantity_received
                product.save()

            inquiry.save()


        print("All inquiries processed and quantity_received updated successfully.")
        
    # If no errors occurred, return success response
    return Response({"message": "Delivery note and dispatch information submitted successfully"}, status=status.HTTP_201_CREATED)

@login_required
def upload_documents_and_rate_driver(request):
     # Filtering quotations where status is either "Produced" or "Dispatched"
    quotations = Quotation.objects.filter(
        Q(status="Produced") | Q(status="Shipped")
    )

    drivers = Driver.objects.all()
    vehicles = Vehicle.objects.all()
    if request.method == 'POST':
        dispatch_product_id = request.POST.get('dispatch_product_id')
        documents = request.FILES.getlist('documents')
        driver_rating = request.POST.get('driver_rating')
        driver_comments = request.POST.get('driver_comments')

        dispatch_product = get_object_or_404(DispatchProduct, pk=dispatch_product_id)
        driver = dispatch_product.driver

        # Handle file uploads
        for document in documents:
            if document.content_type == 'application/pdf':
                # Save the PDF file to the DispatchProduct model
                if dispatch_product.files:
                    dispatch_product.files = dispatch_product.files + ',' + document.name
                else:
                    dispatch_product.files = document.name
                dispatch_product.save() 
            else:
                messages.error(request, 'Only PDF files are allowed.')

        # Update the driver's rating
        driver.rating = driver_rating
        
        # Append new comment to the existing comments list
        if driver_comments:
            comments_list = driver.comments
            comments_list.append(driver_comments)
            driver.comments = comments_list
        driver.save()

        messages.success(request, 'Documents uploaded and driver rated successfully!')
        return render(request, 'sales/dispatch/index.html', {
        'quotations': quotations,
        'drivers': drivers,
        'vehicles': vehicles
    })  # Redirect to the dispatch page

    return render(request, 'sales/dispatch/index.html', {
        'quotations': quotations,
        'drivers': drivers,
        'vehicles': vehicles
    })  # Method Not Allowed

# views.py
from rest_framework import status
from rest_framework.decorators import api_view
from rest_framework.response import Response
from django.contrib import messages
from django.shortcuts import get_object_or_404
from .models import DispatchProduct, Driver

from rest_framework.decorators import api_view
from rest_framework.response import Response
from rest_framework import status
from django.shortcuts import get_object_or_404
from django.contrib import messages 

@api_view(['POST'])
def upload_documents_and_rate_driver_api(request):
    dispatch_product_id = request.data.get('dispatch_product_id')
    documents = request.FILES.getlist('documents')
    driver_rating = request.data.get('driver_rating')
    driver_comments = request.data.get('driver_comments') 
    if not dispatch_product_id:
        return Response({'error': 'Dispatch product ID is required.'}, status=status.HTTP_400_BAD_REQUEST)
 
    dispatch_product = get_object_or_404(Quotation, id=dispatch_product_id)
 
    if dispatch_product.status == "Received":
        return Response({'error': 'This dispatch has already been marked as Received. No further changes are allowed.'}, status=status.HTTP_400_BAD_REQUEST)
 
    dispatch_product.status = "Received"
    dispatch_product.save()

    # # Fetch the Dispatch object and driver
    # dispatch = Dispatch.objects.filter(id=dispatch_product_id).first()
    # driver = dispatch.driver

    # # Save the driver's rating and comments
    # if driver_rating:
    #     DriverRating.objects.create(driver=driver, rating=driver_rating, comments=driver_comments)

    #     # Update the driver's overall rating
    #     driver.update_rating(new_rating=driver_rating)

    # Optionally, handle document uploads here if needed
    # ...

    return Response({'message': 'Documents uploaded and driver rated successfully!'}, status=status.HTTP_201_CREATED)

@login_required
def dispatch_profiling(request, profiling_id):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    drivers = Driver.objects.all()

    if request.method == 'POST':
        profiling = get_object_or_404(ProfilingJob, pk=profiling_id)

        existing_dispatch = Dispatch.objects.filter(profiling_job_id=profiling_id).first()

        if existing_dispatch:
            messages.error(request, "Dispatch entry already exists for this profiling.")
            return redirect('profiling:profiling_list')

        collected_by_name = profiling.customer_signature
        transport_cost = request.POST.get('transport_cost', 0)
        time_date_str = request.POST.get('time_date')
        expected_delivery_date_str = request.POST.get('expected_delivery_date')

        time_date = datetime.strptime(time_date_str, '%Y-%m-%d')
        expected_delivery_date = datetime.strptime(expected_delivery_date_str, '%Y-%m-%d')
        dispatched_by = Account.objects.get(id=request.user.id)

        track_number = get_random_string(length=8, allowed_chars='ABCDEFGHIJKLMNOPQRSTUVWXYZ')

        dispatch = Dispatch(
            profiling_job_id=profiling_id,
            collected_by=collected_by_name,
            transport_cost=transport_cost,
            dispatch_date=time_date,
            expected_delivery_date=expected_delivery_date,
            dispatched_by=dispatched_by,
            track_number=track_number,
            status="Dispatched",
            dispatch_type="Profiling"
        )
        dispatch.save()
        messages.success(request, f"Successfully dispatched profiling job with track number {track_number}.")

        return redirect('profiling:profiling_list')

    return render(request, 'dispatch/dispatch_form.html', {'drivers': drivers, 'profiling_id': profiling_id})


@login_required
def dispatch_detail(request, track_number):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    dispatch_entry = Dispatch.objects.get(track_number=track_number)
    data = {'dispatch_entry': dispatch_entry}
    return render(request, 'dispatch/dispatch_details.html', data)


@login_required
def update_dispatch(request, track_number):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    if request.method == 'POST':
        dispatch_entry = get_object_or_404(Dispatch, track_number=track_number)
        dispatch_entry.status = request.POST.get('dispatchStatus')

        images = request.FILES.getlist('dispatchImages')
        for image in images:
            DispatchImage.objects.create(dispatch=dispatch_entry, image=image)

        dispatch_entry.save()

        quotation_id = dispatch_entry.quotation.qid
        quotation = get_object_or_404(Quotation, id=quotation_id)
        quotation.status = 'Received'
        quotation.save()

        messages.success(request, 'Dispatch updated successfully.')
        return redirect('dispatch:dispatch_index')

    return redirect('dispatch:dispatch_index')


@login_required
def export_dispatch_to_excel(request):
    if not (request.user.is_admin_user or request.user.is_super_user):
        return HttpResponseForbidden("You are not authorized to view this page.")

    response = HttpResponse(content_type='application/ms-excel')
    response['Content-Disposition'] = 'attachment; filename="dispatch_details.xls"'

    wb = xlwt.Workbook(encoding='utf-8')
    ws = wb.add_sheet('Dispatch Details')

    row_num = 0
    font_style = xlwt.XFStyle()
    font_style.font.bold = True

    columns = ['Track Number', 'Driver Name', 'Car Number', 'Transport Mode', 'Quotation ID', 'Expected Delivery Date',
               'Dispatch Date', 'Dispatched By']

    for col_num in range(len(columns)):
        ws.write(row_num, col_num, columns[col_num], font_style)

    font_style = xlwt.XFStyle()
    dispatches = Dispatch.objects.all().values_list(
        'track_number', 'driver_name', 'car_number', 'transport_mode', 'quotation_id', 'expected_delivery_date',
        'dispatch_date', 'dispatched_by__username'
    )

    for dispatch in dispatches:
        row_num += 1
        for col_num in range(len(dispatch)):
            ws.write(row_num, col_num, str(dispatch[col_num]), font_style)

    wb.save(response)
    return response